from .version import __version__
from .tmp import TMP_DIR
from tooldog.biotool_model import Biotool, Informations, Credit, Publication, Documentation,\
                                  Contact, Function, Data, Input, Output, Edam, Operation,\
                                  DataType, Format, Topic



#!/usr/bin/env python3

import logging
import os
import urllib.parse
import urllib.request
import tarfile
from tooldog import TMP_DIR

from .utils import *

LOGGER = logging.getLogger(__name__)


class CodeCollector(object):
    """
    Class to download source code from a https://bio.tools entry
    """

    ZIP_NAME = "tool.zip"
    TAR_NAME = "tool.tar"
    TMP_NAME = "tmp"

    def __init__(self, biotool):
        """
        :param biotool: Biotool object
        :type biotool: :class:`tooldog.biotool_model.Biotool`
        """
        self.biotool = biotool

    def _make_tar(self, file_path, tarname):
        with tarfile.open(tarname, mode='w') as archive:
            archive.add(file_path, arcname=self.ZIP_NAME)

    def _get_from_repository(self, url):
        """
        Get source code from a repository link

        :param url: url of the repository
        :type url: STRING
        """
        # Here we deal with repository, have to use regex to test the url and
        # use appropriate strategy to get the code depending the type of repository
        if "github.com" in url:
            return self._get_from_github(url)
        else:
            LOGGER.warn("The url ({}) is not a Github url".format(url))
            LOGGER.warn("ToolDog only deals with Github repository for the moment...")

    def _get_from_github(self, url):
        try:
            zip_url = os.path.join(url, "archive/master.zip")
            response = urllib.request.urlopen(zip_url)
            data = response.read()

            LOGGER.info('Writing data to zip file...')
            zip_path = os.path.join(TMP_DIR, self.ZIP_NAME)
            tar_path = os.path.join(TMP_DIR, self.TAR_NAME)

            write_to_file(zip_path, data, 'wb')

            LOGGER.info('Making tar...')
            self._make_tar(zip_path, tar_path)

            return tar_path
        except:
            LOGGER.warn('Something went wrong with the following Github repository: {}'.format(zip_url))

    def _get_from_source_code(self, url):
        """
        Get source code from a source code link

        :param url: url of the source code
        :type url: STRING
        """
        return None

    def get_source(self):
        """
        Retrieve source code of the tool using links provided in https://bio.tools
        """
        links = self.biotool.informations.links
        for link in links:
            link_type = link.type.lower().translate(str.maketrans(' ', '_'))
            try:
                source_code = getattr(self, '_get_from_{}'.format(link_type))(link.url)
            except AttributeError:
                LOGGER.warn(link_type + ' link type is not processed yet by ToolDog.')
            if source_code is not None:
                # For the moment, consider that if a source code has been found,
                # we just leave the loop.
                break
        return source_code



#!/usr/bin/env python3

"""
Generation of XML for Galaxy from https://bio.tools based on the Tooldog model using
galaxyxml library.
"""

#  Import  ------------------------------

# General libraries
import os
import copy
import logging

# External libraries
from lxml import etree
import galaxyxml.tool as gxt
import galaxyxml.tool.parameters as gxtp
from galaxyxml.tool.import_xml import GalaxyXmlParser

# Class and Objects
from .edam_to_galaxy import EdamToGalaxy
from tooldog import __version__

#  Constant(s)  ------------------------------

LOGGER = logging.getLogger(__name__)
PARAM_COMMENT = "This parameter has been automatically generated from" \
                " https://bio.tools/tool/%s by ToolDog v" + str(__version__) + "."
FIXME = "FIXME: Please map this parameter to its command line argument."

#  Class(es)  ------------------------------


class GalaxyToolGen(object):
    """
    Class to support generation of XML from :class:`tooldog.biotool_model.Biotool` object.
    """

    def __init__(self, biotool, galaxy_url=None, edam_url=None, mapping_json=None,
                 existing_tool=None):
        """
        Initialize a [Tool] object from galaxyxml with the minimal information
        (a name, an id, a version, a description, the command, the command version
        and a help).

        :param biotool: Biotool object of an entry from https://bio.tools.
        :type biotool: :class:`tooldog.biotool_model.Biotool`
        """
        # Initialize GalaxyInfo
        self.etog = EdamToGalaxy(galaxy_url=galaxy_url, edam_url=edam_url,
                                 mapping_json=mapping_json)
        # Initialize counters for inputs and outputs from bio.tools
        self.input_ct = 0
        self.output_ct = 0
        self.biotool_id = biotool.tool_id
        if existing_tool:
            LOGGER.info("Loading existing XML from " + existing_tool)
            gxp = GalaxyXmlParser()
            self.tool = gxp.import_xml(existing_tool)
            # Add a description if missing from description
            if self.tool.root.find('description').text is None:
                self.tool.root.find('description').text = biotool.description.split('.')[0] + '.'
            # Add information about Tooldog version
            self.tool.add_comment("This tool descriptor has been annotated by ToolDog v" +
                                  __version__)
            # Help if missing or TODO
            if self.tool.help is None:
                self.tool.help = biotool.generate_galaxy_help()
            elif "TODO" in self.tool.help:
                LOGGER.info("TODO has been found in help, content has been replaced.")
                self.tool.help = biotool.generate_galaxy_help()

        else:
            LOGGER.info("Creating new GalaxyToolGen object...")
            # Initialize tool
            #   Get the first sentence of the description only
            description = biotool.description.split('.')[0] + '.'
            self.tool = gxt.Tool(biotool.name, biotool.tool_id, biotool.version,
                                 description, "COMMAND", version_command="COMMAND --version")
            self.tool.help = biotool.generate_galaxy_help()
            #   Add information about Galaxy and EDAM in the XML
            self.tool.add_comment("Information was obtained from the Galaxy instance: " +
                                  self.etog.galaxy_url + " v" +
                                  self.etog.galaxy_version + " and EDAM v" +
                                  self.etog.edam_version)
            # Add information about Tooldog version
            self.tool.add_comment("This tool descriptor has been generated by ToolDog v" +
                                  __version__)

    def add_edam_topic(self, topic):
        """
        Add the EDAM topic to the tool (XML: <edam_topics>).

        :param topic: Topic object.
        :type topic: :class:`tooldog.biotool_model.Topic`
        """
        LOGGER.debug("Adding EDAM topic " + topic.get_edam_id() + " to GalaxyToolGen object.")
        if not hasattr(self.tool, 'edam_topics'):
            # First time we add topics to the tool
            self.tool.edam_topics = gxtp.EdamTopics()
        if not self.tool.edam_topics.has_topic(topic.get_edam_id()):
            self.tool.edam_topics.append(gxtp.EdamTopic(topic.get_edam_id()))

    def add_edam_operation(self, operation):
        """
        Add the EDAM operation to the tool (XML: <edam_operations>).

        :param topic: Operation object.
        :type topic: :class:`tooldog.biotool_model.Operation`
        """
        LOGGER.debug("Adding EDAM operation " + operation.get_edam_id() +
                     " to GalaxyToolGen object.")
        if not hasattr(self.tool, 'edam_operations'):
            # First time we add operations to the tool
            self.tool.edam_operations = gxtp.EdamOperations()
        if not self.tool.edam_operations.has_operation(operation.get_edam_id()):
            self.tool.edam_operations.append(gxtp.EdamOperation(operation.get_edam_id()))

    def add_input_file(self, input_obj):
        """
        Add an input to the tool (XML: <inputs>).

        :param input_obj: Input object.
        :type input_obj: :class:`tooldog.biotool_model.Input`
        """
        LOGGER.debug("Adding input to GalaxyToolGen object...")
        if not hasattr(self.tool, 'inputs'):
            self.tool.inputs = gxtp.Inputs()
        # Build parameter
        self.input_ct += 1
        data_uri = input_obj.data_type.get_edam_id()
        # Give unique name to the input
        name = 'INPUT' + str(self.input_ct)
        # Get all different format for this input
        list_formats = []
        if not input_obj.formats:
            list_formats.append(self.etog.get_datatype(edam_data=data_uri))
        else:
            for format_obj in input_obj.formats:
                format_uri = format_obj.get_edam_id()
                list_formats.append(self.etog.get_datatype(edam_data=data_uri,
                                                           edam_format=format_uri))
        formats = ', '.join(list_formats)
        # Create the parameter
        param = gxtp.DataParam(name, label=input_obj.data_type.term,
                               help=input_obj.description, format=formats)
        # Override the corresponding arguments in the command line
        param.command_line_override = '--' + name + ' $' + name
        # Write comment about this param
        param.node.insert(0, etree.Comment(FIXME))
        param.node.insert(0, etree.Comment(PARAM_COMMENT % (self.biotool_id)))
        # Appends parameter to inputs
        self.tool.inputs.append(param)

    def add_output_file(self, output):
        """
        Add an output to the tool (XML: <outputs>).

        :param output: Output object.
        :type output: :class:`tooldog.biotool_model.Output`
        """
        LOGGER.debug("Adding output to GalaxyToolGen object...")
        if not hasattr(self.tool, 'outputs'):
            self.tool.outputs = gxtp.Outputs()
        # Build parameter
        self.output_ct += 1
        data_uri = output.data_type.get_edam_id()
        # Give unique name to the output
        name = 'OUTPUT' + str(self.output_ct)
        # Get all different format for this output
        list_formats = []
        if not output.formats:
            list_formats.append(self.etog.get_datatype(edam_data=data_uri))
        else:
            for format_obj in output.formats:
                format_uri = format_obj.get_edam_id()
                list_formats.append(self.etog.get_datatype(edam_data=data_uri,
                                                           edam_format=format_uri))
        formats = ', '.join(list_formats)
        # Create the parameter
        param = gxtp.OutputData(name, format=formats, from_work_dir=name +
                                "." + formats.replace('.', '/'))
        param.command_line_override = ''
        # Write comment about this param
        param.node.insert(0, etree.Comment(FIXME))
        param.node.insert(0, etree.Comment(PARAM_COMMENT % (self.biotool_id)))
        self.tool.outputs.append(param)

    def add_citation(self, publication):
        """
        Add publication(s) to the tool (XML: <citations>).

        :param publication: Publication object.
        :type publication: :class:`tooldog.biotool_model.Publication`
        """
        LOGGER.debug("Adding citation to GalaxyToolGen object...")
        if not hasattr(self.tool, 'citations'):
            self.tool.citations = gxtp.Citations()
        # Add citation depending the type (doi, pmid...)
        if publication.doi is not None:
            if not self.tool.citations.has_citation('doi', publication.doi):
                self.tool.citations.append(gxtp.Citation('doi', publication.doi))
        # <citation> only supports doi and bibtex as a type
        elif publication.pmid is not None:
            # self.tool.citations.append(gxtp.Citation('pmid', publication.pmid))
            LOGGER.warn('pmid is not supported by <citation>, citation skipped')
        elif publication.pmcid is not None:
            # self.tool.citations.append(gxtp.Citation('pmcid', publication.pmcid))
            LOGGER.warn('pmcid is not supported by <citation>, citation skipped')

    def write_xml(self, out_file=None, index=None, keep_old_command=False):
        """
        Write CWL to STDOUT or out_file(s).

        :param out_file: path to output file.
        :type out_file: STRING
        :param index: Index in case more than one function is described.
        :type index: INT
        """
        # Copy informations to avoid expension of xml in case we write several XMLs
        export_tool = copy.deepcopy(self.tool)
        # Give XML on STDout
        if out_file is None:
            if index is not None:
                print('########## XML number ' + str(index) + ' ##########')
            LOGGER.info("Writing XML file to STDOUT")
            print(export_tool.export(keep_old_command).decode('utf-8'))
        else:
            # Format name for output file(s)
            if index is not None:
                out_file = os.path.splitext(out_file)[0] + str(index) + '.xml'
            else:
                out_file = os.path.splitext(out_file)[0] + '.xml'
            LOGGER.info("Writing XML file to " + out_file)
            with open(out_file, 'w') as file_w:
                file_w.write(export_tool.export(keep_old_command).decode('utf-8'))



#!/usr/bin/env python3

"""
Generation of CWL tool from https://bio.tools based on the ToolDog model using
cwlgen library.
"""

#  Import  ------------------------------

# General libraries
import os
import logging

# External libraries
import cwlgen
from cwlgen.import_cwl import CWLToolParser

# Class and Objects

#  Constant(s)  ------------------------------

LOGGER = logging.getLogger(__name__)

#  Class(es)  ------------------------------


class CwlToolGen(object):
    """
    Class to support generation of CWL from :class:`tooldog.biotool_model.Biotool` object.
    """

    def __init__(self, biotool, existing_tool=None):
        """
        Initialize a [CommandLineTool] object from cwlgen.

        :param biotool: Biotool object of an entry from https://bio.tools.
        :type biotool: :class:`tooldog.biotool_model.Biotool`
        """
        if existing_tool:
            LOGGER.info("Loading existing CWL tool from " + existing_tool)
            ctp = CWLToolParser()
            self.tool = ctp.import_cwl(existing_tool)
            if 'None' in self.tool.doc:
                self.tool.doc = biotool.generate_cwl_doc()
        else:
            LOGGER.info("Creating new CwlToolGen object...")
            # Initialize counters for inputs and outputs
            self.input_ct = 0
            self.output_ct = 0
            # Initialize tool
            #   Get the first sentence of the description only
            description = biotool.description.split('.')[0] + '.'
            documentation = biotool.generate_cwl_doc()
            self.tool = cwlgen.CommandLineTool(tool_id=biotool.tool_id,
                                               label=description,
                                               base_command="COMMAND",
                                               doc=documentation,
                                               cwl_version='v1.0')
        self._set_meta_from_biotool(biotool)

    def add_input_file(self, input_obj):
        """
        Add an input to the CWL tool.

        :param input_obj: Input object.
        :type input_obj: :class:`tooldog.biotool_model.Input`
        """
        LOGGER.info("Adding input to CwlToolGen object...")
        # Build parameter
        self.input_ct += 1
        # Give unique name to the input
        name = 'INPUT' + str(self.input_ct)
        # Get all different formats for this input
        list_formats = []
        for format_obj in input_obj.formats:
            list_formats.append(format_obj.uri)
        formats = ', '.join(list_formats)
        # Create the parameter
        param_binding = cwlgen.CommandLineBinding(prefix='--' + name)
        param = cwlgen.CommandInputParameter(name, param_type='File',
                                             label=input_obj.data_type.term,
                                             param_format=formats,
                                             input_binding=param_binding)
        # Appends parameter to inputs
        self.tool.inputs.append(param)

    def add_output_file(self, output):
        """
        Add an output to the CWL tool.

        :param output: Output object.
        :type output: :class:`tooldog.biotool_model.Output`
        """
        LOGGER.info("Adding output to CwlToolGen object...")
        # Build parameter
        self.output_ct += 1
        # Give unique name to the output
        name = 'OUTPUT' + str(self.output_ct)
        # Get all different format for this output
        list_formats = []
        for format_obj in output.formats:
            list_formats.append(format_obj.uri)
        formats = ', '.join(list_formats)
        # Create the parameter
        param_binding = cwlgen.CommandOutputBinding(glob=name + '.ext')
        param = cwlgen.CommandOutputParameter(name, param_type='File',
                                              label=output.data_type.term,
                                              param_format=formats,
                                              output_binding=param_binding)
        self.tool.outputs.append(param)

    def _set_meta_from_biotool(self, biotool):
        """
        Add first set of metadata found on bio.tools to the description.

        :param biotool: Biotool object of an entry from https://bio.tools.
        :type biotool: :class:`tooldog.biotool_model.Biotool`
        """
        self.tool.metadata = cwlgen.Metadata()
        self.tool.metadata.name = biotool.name
        self.tool.metadata.about = biotool.description
        self.tool.metadata.url = biotool.homepage
        if biotool.informations.language:
            self.tool.metadata.programmingLanguage = biotool.informations.language

    def add_publication(self, publication):
        """
        Add publication to the tool (CWL: s:publication).

        :param publication: Publication object.
        :type publication: :class:`tooldog.biotool_model.Publication`
        """
        LOGGER.debug("Adding publication to CwlToolGen object...")
        if not hasattr(self.tool.metadata, 'publication'):
            self.tool.metadata.publication = []
        # Add citation depending the type (doi, pmid...)
        if publication.doi is not None:
            self.tool.metadata.publication.append({'id': 'http://dx.doi.org/' + publication.doi})
        # <citation> only supports doi and bibtex as a type
        elif publication.pmid is not None:
            LOGGER.warn('pmid is not supported by publication, publication skipped')
        elif publication.pmcid is not None:
            LOGGER.warn('pmcid is not supported by publication, publication skipped')

    def add_edam_topic(self, topic):
        """
        Add the EDAM topic to the tool (CWL: s:topic).

        :param topic: Topic object.
        :type topic: :class:`tooldog.biotool_model.Topic`
        """
        LOGGER.debug("Adding EDAM topic to CwlToolGen object...")
        if not hasattr(self.tool.metadata, 'keywords'):
            self.tool.metadata.keywords = []
            self.tool.namespaces.edam = "https://edamontology.org/"
        self.tool.metadata.keywords.append('edam:' + topic.get_edam_id())

    def add_edam_operation(self, operation):
        """
        Add the EDAM operation to the tool (CWL: s:operation).

        :param operation: Operation object.
        :type operation: :class:`tooldog.biotool_model.Operation`
        """
        LOGGER.debug("Adding EDAM operation to CwlToolGen object...")
        if not hasattr(self.tool.metadata, 'keywords'):
            self.tool.metadata.keywords = []
            self.tool.namespaces.edam = "https://edamontology.org/"
        self.tool.metadata.keywords.append('edam:' + operation.get_edam_id())

    def write_cwl(self, out_file=None, index=None):
        """
        Write CWL to STDOUT or out_file(s).

        :param out_file: path to output file.
        :type out_file: STRING
        :param index: Index in case more than one function is described.
        :type index: INT
        """
        # Give CWL on STDout
        if out_file is None:
            if index is not None:
                print('########## CWL number ' + str(index) + ' ##########')
            LOGGER.info("Writing CWL file to STDOUT...")
            self.tool.export()
        else:
            # Format name for output file(s)
            if index is not None:
                out_file = os.path.splitext(out_file)[0] + str(index) + '.cwl'
            else:
                out_file = os.path.splitext(out_file)[0] + '.cwl'
            LOGGER.info("Writing CWL file to " + out_file)
            self.tool.export(out_file)



#!/usr/bin/env python3

"""
Wrapper for docker-py low-level client API.
Allow creating container and using it within `with` statement.
"""

# Imports

import docker


# Classes

class Container:
    """
    Class to represent docker container and expose simple API to it.
    """

    def __init__(self, image, command, environment=None):
        """
        Create the container. Not the same as `docker run`, need to be started after the creation.

        :param image: the image to run
        :type image: STRING
        :param command: the command to be run in the container
        :type command: STRING or LIST
        :param environment: A dictionary or a list of strings in the following format {"TEST": "123"} or ["TEST=123"].
        :type environment: DICT or LIST
        :param mount: absolute path to file to mount
        :type mount: STRING
        """
        self.client = docker.APIClient()
        self.pull(image)
        self.id = None
        self.run(image, command, environment)

    def __enter__(self):
        """
        Start the container and make a `with` context.
        """
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        Stop the container and clean up.
        """
        self.stop()
        self.remove()

    def put(self, real_path, container_path):
        with open(real_path, "rb") as bin:
            data = bin.read()
            self.client.put_archive(self.id, container_path, data)

    def run(self, image, command, environment=None):
        """
        Do real work of __init__.
        """
        result = self.client.create_container(
            image,
            command=command,
            detach=True,
            environment=environment,
        )
        self.id = result['Id']

    def exec(self, command):
        """
        Execute the command inside container cid

        :param str command: String representation of bash command

        :return: Returns a generator of output of the result of running bash command in bytes
        :rtype: iter
        """
        exe = self.client.exec_create(container=self.id, cmd=command)
        exe_start = self.client.exec_start(exec_id=exe, stream=True)

        return iter(exe_start)  # as iterator

    def inspect(self):
        """
        :return: Description of the container ans its status
        :rtype: DICT
        """
        return self.client.inspect_container(self.id)

    def start(self):
        """
        Start the container.
        """
        return self.client.start(self.id)

    def stop(self):
        """
        Stop the container.
        """
        return self.client.stop(self.id)

    def logs(self):
        """
        :return: Returns a stream of the STDOUT and STDERR
        :rtype: iter
        """
        return self.client.attach(self.id,
                                  stdout=True,
                                  stderr=True,
                                  stream=True,  # as iterator
                                  logs=False)

    def kill(self):
        """
        Kill the container.
        """
        return self.client.kill(self.id)

    def remove(self):
        """
        Remove the container and associated volumes.
        """
        return self.client.remove_container(self.id, v=True)

    def pull(self, image):
        """
        Pull image
        :param image: the image to pull
        :type image: STRING
        """
        self.client.pull(image)



#!/usr/bin/env python3

"""
Gather different information from a Galaxy server (by default https://usegalaxy.org)
and EDAM ontology (by default from http://edamontology.org/EDAM.owl)
"""

#  Import  ------------------------------

# General libraries
import os
import json
import logging

# External libraries
import requests
import rdflib

#  Constant(s)  ------------------------------

LOCAL_DATA = os.path.dirname(__file__) + "/data"

# Logger
LOGGER = logging.getLogger(__name__)

#  Class(es)  ------------------------------


class GalaxyInfo(object):
    """
    Class to gather different information about a Galaxy instance.

    By default, if the galaxy_url is None, information is loaded from local files
    located in the `data/` folder corresponding to https://usegalaxy.org.
    """

    def __init__(self, galaxy_url):
        """
        :param galaxy_url: URL of the Galaxy instance.
        :type galaxy_url: STRING

        :class:`tooldog.edam_to_galaxy.GalaxyInfo` object is initialized with several
        information from the given Galaxy instance. It contains:

        :param self.version: version of the Galaxy instance.
        :type self.version: STRING
        :param self.edam_formats: mapping edam_format to LIST of extension of datatypes.
        :type self.edam_formats: DICT
        :param self.edam_data: mapping edam_data to LIST of extension of datatypes.
        :type self.edam_data: DICT
        :param self.hierarchy: class_to_classes part of the /api/mapping.json which maps
        the parental classes of each classes.
        :type self.hierarchy: DICT
        :param self.class_names: ext_to_class_name part of the /api/mapping.json which
        maps the extension of a datatype to its class in Galaxy.
        :type self.class_names: DICT
        """
        if galaxy_url is None:
            self.galaxy_url = "https://usegalaxy.org"
            LOGGER.info("Loading Galaxy info (https://usegalaxy.org) from " + LOCAL_DATA)
            with open(LOCAL_DATA + "/edam_formats.json") as json_file:
                api_edam_formats = json.load(json_file)
            with open(LOCAL_DATA + "/edam_data.json") as json_file:
                api_edam_data = json.load(json_file)
            with open(LOCAL_DATA + "/mapping.json") as json_file:
                mapping = json.load(json_file)
            with open(LOCAL_DATA + "/version.json") as json_file:
                version = json.load(json_file)
        else:
            self.galaxy_url = galaxy_url
            LOGGER.info("Loading galaxy info from " + galaxy_url + "/api")
            api_edam_formats = requests.get(galaxy_url + "/api/datatypes/edam_formats").json()
            api_edam_data = requests.get(galaxy_url + "/api/datatypes/edam_data").json()
            mapping = requests.get(galaxy_url + "/api/datatypes/mapping").json()
            version = requests.get(galaxy_url + "/api/version").json()
        # Get version of Galaxy instance
        self.version = version['version_major']

        def rev_dict(dictionnary):
            """
            Reverse dictionnary key -> value to value -> LIST of key
            """
            new_dict = {}
            for key, value in dictionnary.items():
                if value not in new_dict:
                    new_dict[value] = []
                new_dict[value].append(key)
            return new_dict
        self.edam_formats = rev_dict(api_edam_formats)
        self.edam_data = rev_dict(api_edam_data)
        # Store hierarchy from mapping and class names
        self.hierarchy = mapping["class_to_classes"]
        self.class_names = mapping["ext_to_class_name"]

    def select_root(self, datatypes):
        """
        Select the root datatype from all given datatypes.

        :param datatypes: list of different datatypes.
        :type datatypes: list of STRING
        :return: root datatype.
        :rtype: STRING
        """
        # Build class to ext dictionnary
        class_to_ext = {}
        for key, value in self.class_names.items():
            if value not in class_to_ext:
                class_to_ext[value] = []
            class_to_ext[value].append(key)
        # Create subdict of hierarchy
        sub_dict = {}
        for datatype in datatypes:
            if datatype in self.class_names:
                sub_dict[self.class_names[datatype]] = \
                  self.hierarchy[self.class_names[datatype]]
            else:
                LOGGER.warning(datatype + " was not found in the ext to class mapping. skipped")
        # Remove class that inherit from both Binary and Text
        datatype_to_remove = []
        for key, value in sub_dict.items():
            binary = 'galaxy.datatypes.binary.Binary' in value
            text = 'galaxy.datatypes.data.Text' in value
            if binary and text:
                datatype_to_remove.append(key)
        for key in datatype_to_remove:
            del sub_dict[key]
        LOGGER.debug(sub_dict)
        # Find root
        selected_class = None
        root_dist = 100  # Set up huge root distance for comparison
        for key, value in sub_dict.items():
            for key, value in sub_dict.items():
                if len(value) < root_dist:
                    root_dist = len(value)
                    selected_class = key
        if selected_class is None:
            LOGGER.warning("No best datatype found, return first datatype of the list")
            return datatypes[0]
        return class_to_ext[selected_class][0]


class EdamInfo(object):
    """
    Contains the given EDAM ontology.

    It is also possible to generate several dictionnaries to help interrogating the ontology
    for a faster access.
    """

    def __init__(self, edam_url):
        """
        :param edam_url: path to EDAM.owl file
        :type edam_url: STRING

        All the EDAM ontology will be contained in a dictionnary (self.edam_ontology).
        """
        if edam_url is None:
            LOGGER.info("Loading EDAM info from http://edamontology.org/EDAM.owl")
            self.edam_ontology = rdflib.Graph()
            self.edam_ontology.parse("http://edamontology.org/EDAM.owl")
            # Get version of EDAM ontology
            version_query = """SELECT ?version WHERE {
                                     <http://edamontology.org> doap:Version ?version}"""
            for row in self.edam_ontology.query(version_query):
                self.version = row[0]
                break
        else:
            pass

    def generate_hierarchy(self):
        """
        Generates two dictionnaries of the EDAM hierarchy (format and data) with the following
        structure:

        DICT[edam_uri] -> LIST of edam_uri from parents

        The dictionnary can be accessed via self.edam_format_hierarchy
        """

        def make_hierarchy(query):
            """
            Build hierarchy for a given query.

            :return: generated hierarchy
            :rtype: DICT
            """
            hierarchy = {}
            for row in self.edam_ontology.query(query):
                uri = row[0].split('/')[-1]
                p_uri = row[1].split('/')[-1]
                if uri not in hierarchy:
                    hierarchy[uri] = []
                hierarchy[uri].append(p_uri)
            return hierarchy

        formats_query = """SELECT ?format ?superformat WHERE {
                                    ?format rdfs:subClassOf ?superformat .
                                    ?superformat oboInOwl:inSubset
                                   <http://purl.obolibrary.org/obo/edam#formats>
                                    }"""
        data_query = """SELECT ?data ?superdata WHERE {
                                 ?data rdfs:subClassOf ?superdata .
                                 ?superdata oboInOwl:inSubset
                                <http://purl.obolibrary.org/obo/edam#data>
                                 }"""
        self.edam_format_hierarchy = make_hierarchy(formats_query)
        self.edam_data_hierarchy = make_hierarchy(data_query)


class EdamToGalaxy(object):
    """
    Class to make the link between EDAM ontology terms (edam_format and edam_data) and Galaxy
    datatypes.
    """

    def __init__(self, galaxy_url=None, edam_url=None, mapping_json=None):
        """
        :param galaxy_url: URL of the galaxy instance.
        :type galaxy_url: STRING
        :param edam_url: path to EDAM.owl file (URL or local path).
        :type edam_url: STRING
        :param mapping_json: path to personnalized EDAM mapping to Galaxy.
        :type mapping_json: STRING
        """
        if mapping_json is None:
            if galaxy_url or edam_url:
                mapping_json = 'edam_to_galaxy.json'
            else:
                mapping_json = LOCAL_DATA + "/edam_to_galaxy.json"
        # Generates or Loads ?
        if os.path.isfile(mapping_json):
            self.load_local_mapping(mapping_json)
        else:
            # No local file exists, needs to generate it (takes a little bit of time)
            self.edam = EdamInfo(edam_url)
            self.edam_version = self.edam.version
            self.edam.generate_hierarchy()
            self.galaxy = GalaxyInfo(galaxy_url)
            self.galaxy_url = self.galaxy.galaxy_url
            self.galaxy_version = self.galaxy.version
            self.generate_mapping()
            self.export_info(mapping_json)

    def generate_mapping(self):
        """
        Generates mapping between edam_format and edam_data to Galaxy datatypes
        based on the information of the Galaxy instance and the EDAM ontology.

        Every edam_format and edam_data will be given a datatype.
        """
        LOGGER.info("Generating new EDAM mapping to Galaxy datatypes file...")

        def find_datatype(edam, edam_hierarchy, galaxy_mapping):
            """
            Find the best datatype for a given EDAM term.
            :param edam: EDAM term.
            :type edam: STRING
            :param edam_hierarchy: edam_hierarchy from :class:`tooldog.edam_to_galaxy.EdamInfo`
            :type edam_hierarchy: DICT
            :param galaxy_mapping: mapping from :class:`tooldog.edam_to_galaxy.GalaxyInfo`
            :type galaxy_mapping: DICT

            The function then create two dictionnaries: self.format_to_datatype and
            self.data_to_datatype that represents a unique datatype for each EDAM term.
            """
            if edam not in galaxy_mapping:
                LOGGER.debug("No datatype found for " + edam + ". Looking at parental terms.")
                if len(edam_hierarchy[edam]) > 1:
                    LOGGER.debug(edam + " inherits from more than one EDAM. " +
                                 "Only first EDAM parent of the list is treated: " +
                                 edam_hierarchy[edam][0])
                elif len(edam_hierarchy[edam]) == 0:
                    LOGGER.debug("No parental EDAM found. " + edam + " is skipped.")
                    return "NO mapping"
                datatype = find_datatype(edam_hierarchy[edam][0], edam_hierarchy,
                                         galaxy_mapping)
            elif len(galaxy_mapping[edam]) == 1:
                LOGGER.debug("Exactly one datatype found for " + edam + ": " +
                             galaxy_mapping[edam][0])
                datatype = galaxy_mapping[edam][0]
            elif len(galaxy_mapping[edam]) > 1:
                LOGGER.debug("More than one datatypes found for " + edam)
                datatype = self.galaxy.select_root(galaxy_mapping[edam])
            return datatype

        def maps_datatype(edam_hierarchy, galaxy_mapping):
            """
            Maps all edam terms to a Galaxy datatype.
            :param edam_hierarchy: edam_hierarchy from :class:`tooldog.edam_to_galaxy.EdamInfo`
            :type edam_hierarchy: DICT
            :param galaxy_mapping: mapping from :class:`tooldog.edam_to_galaxy.GalaxyInfo`
            :type galaxy_mapping: DICT
            :return: mapping EDAM term to Galaxy datatype (unique mapping).
            :rtype: DICT
            """
            map_to_datatype = {}
            for edam in edam_hierarchy.keys():
                map_to_datatype[edam] = find_datatype(edam, edam_hierarchy, galaxy_mapping)
            return map_to_datatype

        # EDAM formats
        self.format_to_datatype = maps_datatype(self.edam.edam_format_hierarchy,
                                                self.galaxy.edam_formats)
        # EDAM data
        self.data_to_datatype = maps_datatype(self.edam.edam_data_hierarchy,
                                              self.galaxy.edam_data)

    def load_local_mapping(self, local_file):
        """
        Method to load (from JSON file) mapping previously generated and exported in the
        `local_file`.

        :param local_file: path to the mapping local file.
        :type local_file: STRING
        """
        LOGGER.info("Loading EDAM mapping to Galaxy datatypes from " +
                    local_file)
        with open(local_file, 'r') as file_path:
            json_file = json.load(file_path)
        self.format_to_datatype = json_file['format']
        self.data_to_datatype = json_file['data']
        self.galaxy_url = json_file['galaxy_url']
        self.galaxy_version = json_file['galaxy_version']
        self.edam_version = json_file['edam_version']

    def export_info(self, export_file):
        """
        Method to export mapping of this object to a JSON file.

        :param export_file: path to the file.
        :type export_file: STRING
        """
        LOGGER.info("Exporting new EDAM mapping to Galaxy datatypes file to ./" +
                    export_file)
        with open(export_file, 'w') as file_path:
            json.dump({'format': self.format_to_datatype,
                       'data': self.data_to_datatype,
                       'edam_version': self.edam_version,
                       'galaxy_url': self.galaxy_url,
                       'galaxy_version': self.galaxy_version}, file_path)

    def get_datatype(self, edam_data=None, edam_format=None):
        """
        Get datatype from EDAM terms.
        :param edam_data: EDAM data term.
        :type edam_data: STRING
        :param edam_format: EDAM format term.
        :type edam_format: STRING
        :return: datatype corresponding to given EDAM ontologies.
        :rtype: STRING
        """
        if edam_format is not None:
            return self.format_to_datatype[edam_format]
        elif edam_data is not None:
            return self.data_to_datatype[edam_data]
        else:
            return "no EDAM given"



# Set of util functions for work with ToolDog container

import sys
import logging

LOGGER = logging.getLogger(__name__)

def cd(path, cmd):
    return "bash -c 'cd " + path + " && " + cmd + "'"


def pip(v, cmd):
    return "pip" + str(v) + " " + cmd


def execute(ctx, cmd):
    result = ''
    exe = ctx.exec(cmd)
    for line in exe:
        output = line.decode("utf-8")[:-1]
        LOGGER.info(output)
        result += output
    return result


def gen_cmd(tool_name, gen_format):
    return tool_name + " " + ("--generate_cwl_tool" if gen_format == 'cwl' else "--generate_galaxy_xml")


def get_workdir(unzip_output):
    import re
    m = re.search("creating: (.*?)\\n", unzip_output, re.M)
    return "/" + m.group(1).strip()


def if_installed(toolname, output):
    substr = toolname + ": command not found"
    if output.find(substr) == -1:
        return True
    else:
        return False


def tool_filename(tool_name, gen_format):
    return tool_name + "." + ("cwl" if gen_format == 'cwl' else "xml")


def write_to_file(filename=None, data='', mode='w'):
    if filename:
        f = open(filename, mode)
        f.write(data)
        f.close()
    else:
        sys.stdout.write(data)


class DockerException(Exception):
    pass



#!/usr/bin/env python3

import logging
import os
from .container import Container
from .utils import *
from tooldog import TMP_DIR

LOGGER = logging.getLogger(__name__)


class LanguageAnalyzer(object):
    """
    This should be the abstract class for all analyzer.
    """

    def __init__(self, biotool):
        """
        :param biotool: Biotool object
        :type biotool: :class:`tooldog.biotool_model.Biotool`
        """
        self.biotool = biotool

    def analyse(self):
        """
        Run source code analysis.
        """
        pass


class PythonAnalyzer(LanguageAnalyzer):
    """
    Object to specifically analyze Python source code.
    """

    def __init__(self, gen_format, source_code):
        """
        :param gen_format: tool description language (Galaxy XML or CWL)
        :type gen_format: STRING
        :param source_code: path to source code.
        :type source_code: STRING
        """
        self.gen_format = gen_format  # To be used as an option when running Docker
        self.source_code = source_code

    def analyse(self):
        """
        Run source code analysis.
        """

        try:
            LOGGER.info("Trying to analyse the code as python2")
            return self._analyse(2)
        except DockerException:
            try:
                LOGGER.warn("Unable to analyse the code as python2")
                LOGGER.info("Trying to analyse the code as python3")
                return self._analyse(3)
            except DockerException:
                LOGGER.warn("Unable to analyse the code as python3")
                LOGGER.info("Skipping analysis...")
                return None
        except Exception:  # TODO: Add more fine-grained error handlers.
            LOGGER.warn("Unknown Docker client error: Docker is not installed/started or unable to use network due to the network restrictions.")
            LOGGER.info("Skipping analysis...")
            return None

    def _analyse(self, version):
        python_path = "/usr/local/lib/python3.5/dist-packages/" if version == 3 else \
            "/usr/local/lib/python2.7/dist-packages/"

        c = Container("tooldog/analyser",
                      "tail -f /dev/null",  # run until we will stop the container
                      environment={'PYTHONPATH': python_path})

        c.put(self.source_code, "/")

        output = ''
        workdir = ''
        toolname = ''
        with c:
            workdir = get_workdir(execute(c, "unzip /tool.zip"))

            toolname = execute(c, cd(workdir, "python setup.py --name"))

            execute(c,
                    cd(workdir, pip(version, "install .")))
            execute(c,
                    cd(workdir, pip(version, "install git+https://github.com/erasche/argparse2tool")))

            output = execute(c, cd(workdir, gen_cmd(toolname, self.gen_format)))

        if if_installed(toolname, output):
            output_path = os.path.join(TMP_DIR, tool_filename(toolname, self.gen_format))

            write_to_file(output_path, output, 'w')

            return output_path
        else:
            raise DockerException("Tool was not installed properly")



#!/usr/bin/env python3

## Author(s): Kenzo-Hugo Hillion
## Contact(s): kehillio@pasteur.fr
## Python version: 3.6.0
## Creation : 12-19-2016

'''
End to end tests for ToolDog with several examples
'''

###########  Import  ###########

# General libraries
import re
import os
import argparse
import sys
import filecmp
import unittest
from glob import glob

# External libraries
import requests_mock
from nose_parameterized import parameterized

# Class and Objects
from tooldog import main

###########  Constant(s)  ###########

###########  Function(s)  ###########

def get_test_files_xml():
    '''
    List .json files of the test directory for XMLs
    '''
    test_files = []
    json_dir = os.path.dirname(__file__)
    json_list = glob(os.path.join(json_dir, '*.json'))
    for json_path in json_list:
        name = os.path.splitext(json_path)[0]
        xml_path = name + '.xml'
        test_files.append([name, json_path, xml_path])
    return test_files

def get_test_files_cwl():
    '''
    List .json files of the test directory for CWLs
    '''
    test_files = []
    json_dir = os.path.dirname(__file__)
    json_list = glob(os.path.join(json_dir, '*.json'))
    for json_path in json_list:
        name = os.path.splitext(json_path)[0]
        cwl_path = name + '.cwl'
        test_files.append([name, json_path, cwl_path])
    return test_files

###########  Class(es)  ###########

class TestToolDog(unittest.TestCase):

    @parameterized.expand(get_test_files_xml())
    def test_from_local_to_galaxy(self, name, json_path, xml_path):
        json = main.json_from_file(json_path)
        biotool = main.json_to_biotool(json)
        tmp_file = 'tmp_test_xml.xml'
        main.write_xml(biotool,tmp_file)
        tmp_file_list = glob("tmp_*.xml")
        try:
            for temp_file in tmp_file_list:
                if len(tmp_file_list) > 1:
                    xml_path = os.path.splitext(json_path)[0] + \
                               str(re.findall('\d+', temp_file)[0]) + '.xml' 
                self.assertTrue(filecmp.cmp(xml_path,temp_file))
        finally:
            for temp_file in tmp_file_list:
                os.remove(temp_file)

    @parameterized.expand(get_test_files_xml())
    def test_from_biotools_to_galaxy(self, name, json_path, xml_path):
        # Open json to be the content of the requests_mock
        json_answer = main.json_from_file(json_path)
        with requests_mock.mock() as m:
            m.get('https://bio.tools/api/tool/' + name + '/version/1.0',\
                  json=json_answer)
            json = main.json_from_biotools(name, '1.0')
            biotool = main.json_to_biotool(json)
            tmp_file = 'tmp_test_xml.xml'
            main.write_xml(biotool,tmp_file)
            tmp_file_list = glob("tmp_*.xml")
            try:
                for temp_file in tmp_file_list:
                    if len(tmp_file_list) > 1:
                        xml_path = os.path.splitext(json_path)[0] + \
                                   str(re.findall('\d+', temp_file)[0]) + '.xml' 
                    self.assertTrue(filecmp.cmp(xml_path,temp_file))
            finally:
                for temp_file in tmp_file_list:
                    os.remove(temp_file)

    @parameterized.expand(get_test_files_cwl())
    def test_from_local_to_cwl(self, name, json_path, cwl_path):
        json = main.json_from_file(json_path)
        biotool = main.json_to_biotool(json)
        tmp_file = 'tmp_test_cwl.cwl'
        main.write_cwl(biotool,tmp_file)
        tmp_file_list = glob("tmp_*.cwl")
        try:
            for temp_file in tmp_file_list:
                if len(tmp_file_list) > 1:
                    cwl_path = os.path.splitext(json_path)[0] + \
                               str(re.findall('\d+', temp_file)[0]) + '.cwl' 
                self.assertTrue(filecmp.cmp(cwl_path,temp_file))
        finally:
            for temp_file in tmp_file_list:
                os.remove(temp_file)

    @parameterized.expand(get_test_files_cwl())
    def test_from_biotools_to_cwl(self, name, json_path, cwl_path):
        # Open json to be the content of the requests_mock
        json_answer = main.json_from_file(json_path)
        with requests_mock.mock() as m:
            m.get('https://bio.tools/api/tool/' + name + '/version/1.0',\
                  json=json_answer)
            json = main.json_from_biotools(name, '1.0')
            biotool = main.json_to_biotool(json)
            tmp_file = name + '_tmp_test_cwl.cwl'
            main.write_cwl(biotool,tmp_file)
            tmp_file_list = glob(name + "_tmp_*.cwl")
            print (tmp_file_list)
            try:
                for temp_file in tmp_file_list:
                    if len(tmp_file_list) > 1:
                        cwl_path = os.path.splitext(json_path)[0] + \
                                   str(re.findall('\d+', temp_file)[0]) + '.cwl' 
                    self.assertTrue(filecmp.cmp(cwl_path,temp_file))
            finally:
                pass
                for temp_file in tmp_file_list:
                    os.remove(temp_file)


###########  Main  ###########

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

import logging

from .code_collector import CodeCollector
from .language_analyzer import PythonAnalyzer

LOGGER = logging.getLogger(__name__)


class ToolAnalyzer(object):
    """
    Class to perform appropriate source code analysis of a tool.
    """

    def __init__(self, biotool, gen_format, language=None, source_code=None):
        """
        :param biotool: Biotool object
        :type biotool: :class:`tooldog.biotool_model.Biotool`
        :param gen_format: tool descriptor language (Galaxy XML or CWL)
        :type gen_format: STRING
        :param language: language of the tool
        :type language: STRING
        :param source_code: path to source code
        :type source_code: STRING
        """
        self.biotool = biotool
        self.gen_format = gen_format
        self.language = language
        self.source_code = source_code
        self.output = ''

    def _analyse_python(self):
        """
        Perform analysis of Python.
        """
        pa = PythonAnalyzer(self.gen_format, self.source_code)
        return pa.analyse()

    def _analyse_no_language(self):
        """
        Warning message to mention that no language was specified in bio.tools.

        In the future, we can imagine that a code analysis will be perform to check what is
        the coding language.
        """
        LOGGER.warn("Language was not specified for this tool on https://bio.tools. " +
                    "This feature is not processed for the moment.")

    def _analyse_multi_languages(self):
        """
        Warning message to mention that more than one language was given in bio.tools.

        In the future, need to find which language is the main language of the tool (at least
        the one used to run the tool).
        """
        LOGGER.warn("This tool is decribed as using more than one language. " +
                    "This feature is not processed for the moment.")

    def set_language(self):
        """
        Set the language attribute of the object based on the https://bio.tools description.
        """
        language = self.biotool.informations.language
        if len(language) > 1:
            self.language = "multi_languages"
        elif len(language) == 1:
            self.language = language[0]
        else:
            self.language = "no_language"

    def get_source(self):
        """
        Get source code to give to analyzer.
        """
        # At the end of this method, self.source_code should point to directory or archive
        cc = CodeCollector(self.biotool)
        source = cc.get_source()
        if source is not None:
            self.source_code = source

    def run_analysis(self):
        """
        Method to run analysis of source code of the entry.
        """
        output = None
        if self.source_code is None:
            self.get_source()

        if self.source_code is not None:

            if self.language is None:
                self.set_language()
            language = self.language.lower().translate(str.maketrans(' ', '_'))
            try:
                output = getattr(self, '_analyse_{}'.format(language))()
            except AttributeError:
                LOGGER.warn(language + " language is not processed yet by ToolDog.")

        # Need to return the generated code here
        return output



#!/usr/bin/env python3

"""
Main functions used by ToolDog.
"""

#  Import  ------------------------------

# General libraries
import argparse
import os
import sys
import json
import copy
import logging
import shutil

# External libraries
import requests

from tooldog import __version__, Biotool, TMP_DIR
from tooldog.annotate.galaxy import GalaxyToolGen
from tooldog.annotate.cwl import CwlToolGen
from tooldog.analyse.tool_analyzer import ToolAnalyzer


# Constant(s)  ------------------------------

LOG_FILE = os.path.dirname(__file__) + '/tooldog.log'
global LOGGER
LOGGER = logging.getLogger(__name__)  # for tests

#  Function(s)  ------------------------------


def parse_arguments():
    """
    Defines parser for ToolDog.
    """
    parser = argparse.ArgumentParser(description='Generates XML or CWL from bio.tools entry.')
    # Common arguments for analysis and annotations
    parser.add_argument('biotool_entry', help='bio.tools entry from online resource' +
                        ' (ID[/VERSION], e.g. integron_finder/1.5.1 or integron_finder,' +
                        ' the latest version will be fetched in the latter case)' +
                        ' or from local file (ENTRY.json,' +
                        ' e.g. integron_finder.json)')
    ana_or_desc = parser.add_mutually_exclusive_group(required=False)
    ana_or_desc.add_argument('--analyse', dest='ANALYSE', action='store_true',
                             help='run only analysis step of ToolDog.')
    parser.add_argument('--annotate', dest='ANNOTATE', action='store_true',
                        help='run only annotation step of ToolDog.')
    ana_or_desc.add_argument('--existing_desc', dest='ORI_DESC', default=None,
                             help='Existing Tool descriptor that you want to annotate.')
    parser.add_argument('-f', '--file', dest='OUTFILE', help='write in the OUTFILE instead ' +
                        'of STDOUT.')
    parser.add_argument('-v', '--verbose', action='store_true', dest='VERBOSE',
                        help='display info on STDERR.')
    parser.add_argument('--version', action='version', version=__version__,
                        help='show the version number and exit.')
    # Group for the choice of tool descriptor
    choice_desc = parser.add_argument_group('Choice of tool descriptor')
    exc_group = choice_desc.add_mutually_exclusive_group(required=True)
    exc_group.add_argument('-g', '--galaxy', action='store_true',
                           help='generates XML for Galaxy.', dest='GALAXY')
    exc_group.add_argument('-c', '--cwl', action='store_true', help='generates CWL tool ' +
                           'descriptor.', dest='CWL')
    # Group for analysis options
    analy_opt = parser.add_argument_group('Options for source code analysis')
    analy_opt.add_argument('--source_language', dest='LANG',
                           help='Language of the tool.')
    analy_opt.add_argument('--source_code', dest='SOURCE',
                           help='Path the source code directory.')
    # Group for annotation options
    annot_opt = parser.add_argument_group('Options for tool description annotation')
    annot_opt.add_argument('--inout_biotools', action='store_true',
                           help='add inputs and outputs described in bio.tools to the description',
                           dest='INOUT_BIOT', default=False)
    # Group for Galaxy options
    galaxy_opt = parser.add_argument_group('Options for Galaxy XML generation (-g/--galaxy)')
    galaxy_opt.add_argument('--galaxy_url', dest='GAL_URL',
                            help='url of the Galaxy instance (default: https://usegalaxy.org' +
                            ' ).')
    galaxy_opt.add_argument('--edam_url', dest='EDAM_URL',
                            help='EDAM.owl file either online url or local path ' +
                            '(default: http://edamontology.org/EDAM.owl).')
    galaxy_opt.add_argument('--mapping_file', dest='MAP_FILE',
                            help='Personalized EDAM to datatypes mapping json file ' +
                            'generated previously by ToolDog.')
    # Group for logger options
    log_group = parser.add_argument_group('Logs options')
    log_group.add_argument('-l', '--logs', action='store_true',
                           help='Write logs in tooldog_activity.log.',
                           dest='LOGS')
    log_group.add_argument('--log_level', dest='LOG_LEVEL', default='WARN',
                           help='set up the level of the logger.')
    log_group.add_argument('--log_file', dest='LOG_FILE', default='tooldog_activity.log',
                           help='define an output LOG_FILE.')

    try:
        return parser.parse_args()
    except SystemExit:
        sys.exit(1)


def config_logger(write_logs, log_level, log_file, verbose):
    """
    Initialize the logger for ToolDog. By default, only WARNING, ERROR and CRITICAL are
    written on STDERR. You can also write logs to a log file.

    :param write_logs: Decide to write logs to output log file.
    :type write_logs: BOOLEAN
    :param log_level: Select the level of logs. 'debug', 'info' or 'warn'. Other value\
    is considered as 'warn'.
    :type log_level: STRING
    :param log_file: path to output log file.
    :type log_file: STRING

    :return: Config dictionnary for logger.
    :rtype: DICT
    """
    cfg = {'version': 1,
           'formatters': {'written': {'format': '%(asctime)s :: %(name)s ' +
                                                ':: %(levelname)s :: %(message)s'},
                          'printed': {'format': '%(name)s :: ' +
                                      '%(levelname)s :: %(message)s'}},
           'handlers': {},
           'loggers': {}}
    # Configure handler for all logs if user specified so
    if write_logs:
        cfg_logfile = {'class': 'logging.handlers.RotatingFileHandler',
                       'formatter': 'written',
                       'maxBytes': 1000000,
                       'backupCount': 1}
        cfg_logfile['level'] = log_level
        cfg_logfile['filename'] = log_file
        cfg['handlers']['logfile'] = cfg_logfile
    # Configure handler for Errors, warnings on stderr
    cfg_stderr = {'class': 'logging.StreamHandler',
                  'formatter': 'printed'}
    cfg_stderr['level'] = 'WARNING'
    if verbose:
        cfg_stderr['level'] = 'INFO'
    cfg['handlers']['stderr'] = cfg_stderr
    # Configure loggers for everymodule
    modules = ['annotate.galaxy', 'annotate.cwl', 'annotate.edam_to_galaxy',
               'analyse', 'analyse.tool_analazer', 'analyse.code_collector',
               'analyse.language_analyzer', 'biotool_model', 'main', 'analyse']
    logger = {'handlers': ['stderr'],
              'propagate': False,
              'level': 'DEBUG'}
    if write_logs:
        logger['handlers'].append('logfile')
    for module in modules:
        cfg['loggers']['tooldog.' + module] = logger
    return cfg


def json_from_biotools(tool_id, tool_version="latest"):
    """
    Import JSON of a tool from https://bio.tools.

    :param tool_id: ID of the tool.
    :type tool_id: STRING
    :param tool_version: Version of the tool.
    :type tool_version: STRING

    :return: dictionnary corresponding to the JSON from https://bio.tools.
    :rtype: DICT
    """
    LOGGER.info("Loading tool entry from https://bio.tools: " + tool_id + '/' + tool_version)
    biotools_link = "https://bio.tools/api/tool/" + tool_id + ("/version/" + tool_version if tool_version != "latest" else "/")
    # Access the entry with requests and get the JSON part
    http_tool = requests.get(biotools_link)
    json_tool = http_tool.json()
    if len(json_tool.keys()) == 1:
        # The content of JSON only contains one element which is the results we obtain
        # on bio.tools when an entry does not exist.
        LOGGER.error('Entry not found on https://bio.tools.com. Exit.')
        sys.exit(1)
    return json_tool


def json_from_file(json_file):
    """
    Import JSON of a tool from a local JSON file.

    :param json_file: path to the file
    :type json_file: STRING

    :return: dictionnary corresponding to the JSON.
    :rtype: DICT
    """
    LOGGER.info("Loading tool entry from local file: " + json_file)
    # parse file in JSON format
    with open(json_file, 'r') as tool_file:
        json_tool = json.load(tool_file)
    return json_tool


def json_to_biotool(json_file):
    """
    Takes JSON file from bio.tools description and loads its content to
    :class:`tooldog.model.Biotool` object.

    :param json_file: dictionnary of JSON file from bio.tools description.
    :type json_file: DICT

    :return: Biotool object.
    :rtype: :class:`tooldog.biotool_model.Biotool`
    """
    LOGGER.info("Converting biotool entry (JSON) to Biotool object...")
    # Initialize Biotool object with basic parameters
    biotool = Biotool(json_file['name'], json_file['id'], json_file['version'],
                      json_file['description'], json_file['homepage'])
    # Add informations
    biotool.set_informations(json_file['credit'], json_file['contact'],
                             json_file['publication'], json_file['documentation'],
                             json_file['language'], json_file['link'], json_file['download'])
    # Add Function(s)
    biotool.add_functions(json_file['function'])
    # Add Topics(s)
    biotool.add_topics(json_file['topic'])
    return biotool


def write_xml(biotool, outfile=None, galaxy_url=None, edam_url=None, mapping_json=None,
              existing_tool=None, inout_biotool=False):
    """
    This function uses :class:`tooldog.galaxy.GalaxyToolGen` to write XML
    using galaxyxml library.

    :param biotool: Biotool object.
    :type biotool: :class:`tooldog.biotool_model.Biotool`
    :param outfile: path to output file to write the XML.
    :type outfile: STRING
    :param galaxy_url: link to galaxy instance.
    :type galaxy_url: STRING
    :param edam_url: link to EDAM owl.
    :type edam_url: STRING
    :param mapping_json: local JSON mapping between EDAM and Galaxy datatypes.
    :type mapping_json: STRING
    :param existing_tool: local path to existing Galaxy XML tool description.
    :type existing_tool: STRING
    :param inout_biotool: add input and outputs description from https://bio.tools.
    :type inout_biotool: BOOLEAN
    """
    LOGGER.info("Writing XML file with galaxy.py module...")
    biotool_xml = GalaxyToolGen(biotool, galaxy_url=galaxy_url, edam_url=edam_url,
                                mapping_json=mapping_json, existing_tool=existing_tool)
    # Add EDAM annotation and citations
    for topic in biotool.topics:
        biotool_xml.add_edam_topic(topic)
    for function in biotool.functions:
        for operation in function.operations:
            biotool_xml.add_edam_operation(operation)
    for publi in biotool.informations.publications:
        biotool_xml.add_citation(publi)
    # Add inputs and outputs
    if existing_tool:
        if inout_biotool:
            for function in biotool.functions:
                for inpt in function.inputs:
                    biotool_xml.add_input_file(inpt)
                for output in function.outputs:
                    biotool_xml.add_output_file(output)
        biotool_xml.write_xml(out_file=outfile, keep_old_command=True)
    else:
        # This will need to be changed when incorporating argparse2tool...
        for function in biotool.functions:
            # First make a copy of the tool to add function infos
            function_xml = copy.deepcopy(biotool_xml)
            for inpt in function.inputs:
                function_xml.add_input_file(inpt)
            for output in function.outputs:
                function_xml.add_output_file(output)
            # Write tool
            if len(biotool.functions) > 1:
                function_xml.write_xml(outfile, biotool.functions.index(function) + 1)
            else:
                function_xml.write_xml(outfile)


def write_cwl(biotool, outfile=None, existing_tool=None):
    """
    This function uses :class:`tooldog.cwl.CwlToolGen` to write CWL using cwlgen.
    CWL is generated on STDOUT by default.

    :param biotool: Biotool object.
    :type biotool: :class:`tooldog.biotool_model.Biotool`
    :param outfile: path to output file to write the CWL.
    :type outfile: STRING
    :param existing_tool: local path to existing CWL tool description.
    :type existing_tool: STRING
    """
    LOGGER.info("Writing CWL file with cwl.py module...")
    biotool_cwl = CwlToolGen(biotool, existing_tool=existing_tool)
    # Add different Metadata
    for topic in biotool.topics:
        biotool_cwl.add_edam_topic(topic)
    for publi in biotool.informations.publications:
        biotool_cwl.add_publication(publi)
    if existing_tool:
        # For the moment, there is no way to add metadata to the cwl
        biotool_cwl.write_cwl(outfile)
    else:
        for function in biotool.functions:
            # First make a copy of the tool to add function infos
            function_cwl = copy.deepcopy(biotool_cwl)
            for inp in function.inputs:
                function_cwl.add_input_file(inp)
            for outp in function.outputs:
                function_cwl.add_output_file(outp)
            # Write tool
            if len(biotool.functions) > 1:
                function_cwl.write_cwl(outfile, biotool.functions.index(function) + 1)
            else:
                function_cwl.write_cwl(outfile)


def annotate(biotool, args, existing_desc=None):
    """
    Run annotation (generated by analysis or existing_desc).

    :param biotool: Biotool object.
    :type biotool: :class:`tooldog.biotool_model.Biotool`
    :param args: Parsed arguments.
    :type args: :class:`argparse.ArgumentParser`
    :param existing_desc: Existing tool descriptor path.
    :type existing_desc: STRING
    """
    if args.GALAXY:
        # Probably need to check if existing_desc right format
        write_xml(biotool, outfile=args.OUTFILE, galaxy_url=args.GAL_URL,
                  edam_url=args.EDAM_URL, mapping_json=args.MAP_FILE,
                  existing_tool=existing_desc, inout_biotool=args.INOUT_BIOT)
    elif args.CWL:
        # Write corresponding CWL
        write_cwl(biotool, args.OUTFILE, existing_tool=existing_desc)


def analyse(biotool, args):
    """
    Run analysis of the source code from bio.tools or given locally.

    :param biotool: Biotool object.
    :type biotool: :class:`tooldog.biotool_model.Biotool`
    :param args: Parsed arguments.
    :type args: :class:`argparse.ArgumentParser`
    """
    LOGGER.warn("Analysis feature is in beta version.")
    output = ''
    # Instantiate ToolAnalyzer object
    if args.GALAXY:
        ta = ToolAnalyzer(biotool, 'galaxy', language=args.LANG, source_code=args.SOURCE)
    else:
        ta = ToolAnalyzer(biotool, 'cwl', language=args.LANG, source_code=args.SOURCE)
    # Run analysis
    output = ta.run_analysis()  # Here it depends on how the method works
    # Return path to generated file / descriptor
    return output


def run():
    """
    Running function called by tooldog command line.
    """

    try:
        # Parse arguments
        args = parse_arguments()

        # Logger configuration
        import logging.config
        logging.config.dictConfig(config_logger(args.LOGS, args.LOG_LEVEL,
                                                args.LOG_FILE, args.VERBOSE))
        # Reset LOGGER with new config
        LOGGER = logging.getLogger(__name__)

        # Get JSON of the tool
        if '.json' in args.biotool_entry:
            # Importation from local file
            json_tool = json_from_file(args.biotool_entry)
        elif ('/' in args.biotool_entry) and (len(args.biotool_entry.split('/')) == 2):
            # Importation from https://bio.tools
            tool_ids = args.biotool_entry.split('/')
            json_tool = json_from_biotools(tool_ids[0], tool_ids[1])
        else:
            json_tool = json_from_biotools(args.biotool_entry)
            # Wrong argument given for the entry
            # LOGGER.error('biotool_entry does not have the correct syntax. Exit')
            # parser.print_help()
            # sys.exit(1)

        # Load Biotool object
        biotool = json_to_biotool(json_tool)

        if args.ORI_DESC:
            annotate(biotool, args, args.ORI_DESC)
        elif args.ANALYSE and not args.ANNOTATE:
            analyse(biotool, args)
        elif args.ANNOTATE and not args.ANALYSE:
            annotate(biotool, args)
        else:
            # analyse(biotool, args)
            gen_tool = analyse(biotool, args)
            # The existing_tool need to be changed to what will be generated by analyse().
            annotate(biotool, args, gen_tool)
    finally:
        shutil.rmtree(TMP_DIR)


if __name__ == "__main__":
    run()



#! /usr/bin/env python3

"""
Model used to process information contained in JSON from https://bio.tools description.

The content of a description on https://bio.tools is contained in a JSON file and this
model aims to store the different information.
"""

import requests
import logging
from lxml import etree
from ruamel.yaml.scalarstring import PreservedScalarString

LOGGER = logging.getLogger(__name__)

#  Class(es)  ------------------------------


class Biotool(object):
    '''
    This class correspond to an entry from https://bio.tools.
    '''

    def __init__(self, name, tool_id, version, description, homepage):
        '''
        :param name: Name of the tool.
        :type name: STRING
        :param tool_id: ID of the tool entry.
        :type tool_id: STRING
        :param version: Version of the tool entry.
        :type version: STRING
        :param description: Description of the tool entry.
        :type description: STRING
        :param homepage: URL to homepage.
        :type homepage: STRING

        :class:`tooldog.biotool_model.Biotool` object is also initialized with two empty
        list of objects:

        * functions: list of :class:`tooldog.biotool_model.Function`
        * topics: list of :class:`tooldog.biotool_model.Topic`

        More information (:class:`tooldog.biotool_model.Informations` object) can be specified
        using :meth:`tooldog.biotool_model.Biotool.set_informations`.
        '''
        self.name = name
        self.tool_id = tool_id
        self.version = version
        self.description = description
        self.homepage = homepage
        self.functions = []  # List of Function objects
        self.topics = []    # List of Topic objects
        self.informations = Informations()  # Informations object
        if self.homepage.startswith('https://github.com'):
            link = Link({'url': self.homepage, 'type': 'Repository', 'comment': ''})
            self.informations.links.append(link)

    def generate_galaxy_help(self):
        """
        Generate a help message from the different informations found on the tool.

        :return: a help message for Galaxy XML.
        :rtype: STRING
        """
        help_message = "\n\nWhat it is ?\n" + "============\n\n"
        help_message += self.description + "\n\n"
        help_message += "External links:\n" + "===============\n\n"
        help_message += "- Tool homepage_\n"
        help_message += "- bio.tools_ entry\n\n"
        help_message += ".. _homepage: " + self.homepage + "\n"
        help_message += ".. _bio.tools: https://bio.tools/tool/" + self.tool_id
        return help_message

    def generate_cwl_doc(self):
        """
        Generate a doc from the different informations found on the tool.

        :return: a doc for CWL tool description.
        :rtype: STRING
        """
        doc_message = self.description + "\n\n"
        doc_message += "External links:\n"
        doc_message += "Tool homepage: " + self.homepage + "\n"
        doc_message += "bio.tools entry: " + self.tool_id + "\n\n"
        return doc_message

    def set_informations(self, tool_credits, contacts, publications, docs,
                         language, links, download):
        '''
        Add an :class:`tooldog.biotool_model.Informations` object to the Biotool.

        :param tool_credits: list of different tool_credits.
        :type tool_credits: LIST of DICT
        :param contacts: list of different contacts.
        :type contacts: LIST of DICT
        :param publications: list of different IDs for publications.
        :type publications: LIST of DICT
        :param doc: list of different documentations.
        :type doc: LIST of DICT
        '''
        for cred in tool_credits:
            self.informations.tool_credits.append(Credit(cred))
        for cont in contacts:
            self.informations.contacts.append(Contact(cont))
        for pub in publications:
            self.informations.publications.append(Publication(pub))
        for doc in docs:
            self.informations.documentations.append(Documentation(doc))
        self.informations.language = language
        for link in links:
            self.informations.links.append(Link(link))
        for link in download:
            self.informations.links.append(Link(link))

    def add_functions(self, functions):
        '''
        Add :class:`tooldog.biotool_model.Function` objects to the list of functions of the
        Biotool object.

        :param functions: list of functions description from https://bio.tools.
        :type functions: LIST of DICT
        '''
        for fct in functions:
            # Create Function object
            function = Function(fct['operation'])
            function.add_inputs(fct['input'])
            function.add_outputs(fct['output'])
            # Append object to the biotool
            self.functions.append(function)

    def add_topics(self, topics):
        '''
        Add :class:`tooldog.biotool_model.Topic` objects to the list of topics of the
        Biotool object.

        :param topics: list of topics description from https://bio.tools.
        :type topics: LIST of DICT
        '''
        for topic in topics:
            self.topics.append(Topic(topic))


class Informations(object):
    '''
    Class to describe different information concerning a bio.tool entry.
    '''

    def __init__(self):
        '''
        :class:`tooldog.biotool_model.Informations` object is initialized with four empty
        list of objects:

        * publications: list of :class:`tooldog.biotool_model.Publication`
        * documentations: list of :class:`tooldog.biotool_model.Documentation`
        * contacts: list of :class:`tooldog.biotool_model.Contact`
        * tool_credits: list of :class:`tooldog.biotool_model.Credit`
        * language: list of coding language
        * link: list of :class:`tooldog.biotool_model.Link`
        '''
        self.publications = []
        self.documentations = []
        self.contacts = []
        self.tool_credits = []
        self.language = []
        self.links = []


class Link(object):
    '''
    Class to store download and links content.
    '''

    def __init__(self, link):
        '''
        :param link: links or download content of the JSON from http://bio.tools.
        :type link: DICT
        '''
        self.url = link['url']
        self.type = link['type']
        self.comment = link['comment']


class Credit(object):
    '''
    Class to store a credit information.
    '''

    def __init__(self, credit):
        '''
        :param credit: credit part of the JSON from http://bio.tools.
        :type credit: DICT
        '''
        self.comment = credit['comment']  # [STRING]
        self.email = credit['email']  # [STRING]
        self.grid_id = credit['gridId']  # [STRING]
        self.name = credit['name']  # [STRING]
        self.type_entity = credit['typeEntity']  # [STRING]
        self.type_role = credit['typeRole']  # [STRING]
        self.url = credit['url']  # [STRING]
        self.orcid_id = credit['orcidId']  # [STRING]


class Publication(object):
    '''
    Class to store one publication information.
    '''

    def __init__(self, publication):
        '''
        :param publication: publication part of the JSON from http://bio.tools.
        :type publication: DICT
        '''
        self.doi = publication['doi']  # [STRING]
        self.pmid = publication['pmid']  # [STRING]
        self.pmcid = publication['pmcid']  # [STRING]
        self.type = publication['type']  # [STRING]
        if self.doi is None:
            self._fetch_doi()

    def _fetch_doi(self):
        """
        fetch doi using pmid or pmcid using:
        https://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0
        """
        if self.pmid is not None:
            id_query = self.pmid
        elif self.pmcid is not None:
            id_query = self.pmcid
        req = "https://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/?ids=" + id_query
        xml_req = etree.fromstring(requests.get(req).text)
        if xml_req.find('record') is not None:
            try:
                self.doi = xml_req.find('record').attrib['doi']
            except:
                LOGGER.warning("Could not find doi corresponding to " + id_query)


class Documentation(object):
    '''
    Class to store one documentation information.
    '''

    def __init__(self, documentation):
        '''
        :param documentation: documentation part of the JSON from http://bio.tools.
        :type documentation: DICT
        '''
        self.url = documentation['url']  # [STRING]
        self.type = documentation['type']  # [STRING]
        self.comment = documentation['comment']  # [STRING]


class Contact(object):
    '''
    Class to store one contact information.
    '''

    def __init__(self, contact):
        '''
        :param contact: contact part of the JSON from http://bio.tools.
        :type contact: DICT
        '''
        self.email = contact['email']  # [STRING]
        self.name = contact['name']  # [STRING]
        # self.role = contact['contactRole']
        # self.tel = contact['contactTel']
        # self.url = contact['contactURL']


class Function(object):
    '''
    Correspond to one function of the entry with the corresponding inputs and outputs.
    '''

    def __init__(self, edams):
        '''
        :param edams: EDAM ontology for operation(s) with uri and term.
        :type edams: LIST of DICT
        :class:`tooldog.biotool_model.Function` object is initialized with two empty
        list of objects:

        * inputs: list of :class:`tooldog.biotool_model.Input`
        * outputs: list of :class:`tooldog.biotool_model.Output`
        '''
        self.operations = []
        for edam in edams:
            self.operations.append(Operation(edam))
        self.inputs = []
        self.outputs = []

    def add_inputs(self, inputs):
        '''
        Add inputs to the :class:`tooldog.biotool_model.Function` object.

        :param inputs: inputs part of one function from http://bio.tools.
        :type inputs: LIST of DICT
        '''
        for inp in inputs:
            # Create Input object and appends to the list
            self.inputs.append(Input(inp['data'], inp['format']))

    def add_outputs(self, outputs):
        '''
        Add outputs to the :class:`tooldog.biotool_model.Function` object.

        :param outputs: inputs part of one function from http://bio.tools.
        :type outputs: LIST of DICT
        '''
        for outp in outputs:
            # Create Output object and appends to the list
            self.outputs.append(Output(outp['data'], outp['format']))


class Data(object):
    '''
    Data described by EDAM ontology.
    '''

    def __init__(self, data_type, formats, description=None):
        '''
        :param data_type: EDAM ontology for the data type with uri and term.
        :type data_type: DICT
        :param formats: EDAM ontology for data formats with uri and term.
        :type formats: LIST of DICT
        :param description: description of the data (DEPRECATED)
        :type description: STRING
        '''
        self.data_type = DataType(data_type)
        self.formats = []
        for frmt in formats:
            self.formats.append(Format(frmt))
        self.description = description


class Input(Data):
    '''
    Input of a described function.
    '''

    def __init__(self, data_type, formats, description=None):
        '''
        :param data_type: EDAM ontology for the data type with uri and term.
        :type data_type: DICT
        :param formats: EDAM ontology for data formats with uri and term.
        :type formats: LIST of DICT
        :param description: description of the data (DEPRECATED)
        :type description: STRING
        '''
        Data.__init__(self, data_type, formats, description)


class Output(Data):
    '''
    Output of a described function.
    '''

    def __init__(self, data_type, formats, description=None):
        '''
        :param data_type: EDAM ontology for the data type with uri and term.
        :type data_type: DICT
        :param formats: EDAM ontology for data formats with uri and term.
        :type formats: LIST of DICT
        :param description: description of the data (DEPRECATED)
        :type description: STRING
        '''
        Data.__init__(self, data_type, formats, description)


class Edam(object):
    '''
    Edam annotation with the uri and its corresponding term.
    '''

    def __init__(self, edam):
        '''
        :param edam: EDAM ontology with uri and term.
        :type edam: DICT
        '''
        self.uri = edam['uri']
        self.term = edam['term']

    def get_edam_id(self):
        '''
        Get the EDAM id from the uri.

        :return: EDAM id from the uri.
        :rtype: STRING
        '''
        return self.uri.split('/')[-1]


class Operation(Edam):
    '''
    EDAM operation associated to a function.
    '''

    def __init__(self, edam):
        '''
        :param edam: EDAM ontology with uri and term.
        :type edam: DICT
        '''
        Edam.__init__(self, edam)


class DataType(Edam):
    '''
    EDAM data associated to either input or output.
    '''

    def __init__(self, edam):
        '''
        :param edam: EDAM ontology with uri and term.
        :type edam: DICT
        '''
        Edam.__init__(self, edam)


class Format(Edam):
    '''
    EDAM format associated to either input or output.
    '''

    def __init__(self, edam):
        '''
        :param edam: EDAM ontology with uri and term.
        :type edam: DICT
        '''
        Edam.__init__(self, edam)


class Topic(Edam):
    '''
    EDAM topic associated to the entry.
    '''

    def __init__(self, edam):
        '''
        :param edam: EDAM ontology with uri and term.
        :type edam: DICT
        '''
        Edam.__init__(self, edam)



from setuptools import setup
import sys, os

exec(open('tooldog/version.py').read())

setup(name="tooldog",
        version=__version__,
        description='Tool description generator (from https//bio.tools to XML and CWL)',
        author='Kenzo-Hugo Hillion, Ivan Kuzmin and Herve Menager',
        author_email='kehillio@pasteur.fr and hmenager@pasteur.fr',
        license='MIT',
        keywords = ['biotools','galaxy','xml','cwl'],
        install_requires=['rdflib', 'requests', 'galaxyxml', 'cwlgen>=0.2.3', 'docker==2.1.0'],
        packages=["tooldog", "tooldog.annotate", "tooldog.analyse"],
        package_data={
        'tooldog': ['annotate/data/*'],
        },
        entry_points={'console_scripts':['tooldog=tooldog.main:run']},
        classifiers=[
            'Development Status :: 4 - Beta',
            'Topic :: Scientific/Engineering :: Bio-Informatics',
            'Operating System :: OS Independent',
            'Intended Audience :: Developers',
            'Intended Audience :: Science/Research',
            'Environment :: Console',
            ],
        )



#!/usr/bin/env python3

'''
Unit tests for ToolDog
'''

#  Import  ------------------------------

# General libraries
import os
import json
import filecmp
import unittest

# External libraries
import requests_mock

# Class and Objects
from tooldog import main, biotool_model
from tooldog.annotate import galaxy, cwl, edam_to_galaxy

#  Constant(s)  ------------------------------

# Declare one ontology for all the test
EDAM_TOPIC = {'uri':'http://edamontology.org/topic_0091',
              'term':'bioinformatics'}
EDAM_OPE = {'uri':'http://edamontology.org/operation_2429',
             'term':'mapping'}
EDAM_DATA = {'uri':'http://edamontology.org/data_0924',
             'term':'sequence_trace'}
EDAM_FORMAT = {'uri':'http://edamontology.org/format_1930',
               'term':'fastq'}

#  Function(s)  ------------------------------

#  Class(es)  ------------------------------

class TestBiotool(unittest.TestCase):

    def setUp(self):
        self.biotool = biotool_model.Biotool('name', 'an_id', 'a_version', 'a description '+\
                                       'with spaces.', 'http://urltohomepage.com')

    def test_init(self):
        self.assertEqual(self.biotool.name, 'name')
        self.assertEqual(self.biotool.tool_id, 'an_id')
        self.assertEqual(self.biotool.version, 'a_version')
        self.assertEqual(self.biotool.description, 'a description with spaces.')
        self.assertEqual(self.biotool.homepage, 'http://urltohomepage.com')
        self.assertListEqual(self.biotool.functions, [])
        self.assertListEqual(self.biotool.topics, [])
        #self.assertIsNone(self.biotool.informations)

    def test_set_informations(self):
        tool_credits = [{'comment':'a_comment', 'email':'an_email',\
                    'gridId':'a_grid_id', 'name':'a_name',\
                    'typeEntity':'a_type_entity', 'typeRole':'a_type_role',\
                    'url':'a_url', 'orcidId':'an_orcid_id'}]
        contacts = [{'email':'an_email', 'name':'a_name'}]
        pubs = [{'doi':'a_doi', 'pmid':'a_pm_id', 'pmcid':'a_pmc_id', 'type':'a_type'}]
        docs = [{'url':'an_url', 'type':'a_type', 'comment':'a_comment'}]
        language = ['Python']
        link = [{'url': 'a_url', 'type': 'Repository', 'comment': 'a comment'}]
        download = [{'url': 'a_url', 'type': 'Source code', 'comment': 'a comment'}]
        self.biotool.set_informations(tool_credits, contacts, pubs, docs, language, link, download)
        # Check credits params
        self.assertEqual(self.biotool.informations.tool_credits[0].comment, 'a_comment')
        self.assertEqual(self.biotool.informations.tool_credits[0].email, 'an_email')
        self.assertEqual(self.biotool.informations.tool_credits[0].grid_id, 'a_grid_id')
        self.assertEqual(self.biotool.informations.tool_credits[0].name, 'a_name')
        self.assertEqual(self.biotool.informations.tool_credits[0].type_entity, 'a_type_entity')
        self.assertEqual(self.biotool.informations.tool_credits[0].type_role, 'a_type_role')
        self.assertEqual(self.biotool.informations.tool_credits[0].url, 'a_url')
        self.assertEqual(self.biotool.informations.tool_credits[0].orcid_id, 'an_orcid_id')
        # Check contacts params
        self.assertEqual(self.biotool.informations.contacts[0].email, 'an_email')
        self.assertEqual(self.biotool.informations.contacts[0].name, 'a_name')
        # Check publications params
        self.assertEqual(self.biotool.informations.publications[0].doi, 'a_doi')
        self.assertEqual(self.biotool.informations.publications[0].pmid, 'a_pm_id')
        self.assertEqual(self.biotool.informations.publications[0].pmcid, 'a_pmc_id')
        self.assertEqual(self.biotool.informations.publications[0].type, 'a_type')
        # Check documentations params
        self.assertEqual(self.biotool.informations.documentations[0].url, 'an_url')
        self.assertEqual(self.biotool.informations.documentations[0].type, 'a_type')
        self.assertEqual(self.biotool.informations.documentations[0].comment, 'a_comment')
        # Check language params
        self.assertEqual(self.biotool.informations.language[0], 'Python')
        # Check links
        self.assertEqual(self.biotool.informations.links[0].url, 'a_url')
        self.assertEqual(self.biotool.informations.links[0].type, 'Repository')
        self.assertEqual(self.biotool.informations.links[0].comment, 'a comment')
        self.assertEqual(self.biotool.informations.links[1].url, 'a_url')
        self.assertEqual(self.biotool.informations.links[1].type, 'Source code')
        self.assertEqual(self.biotool.informations.links[1].comment, 'a comment')

    def test_add_functions(self):
        function = [{'operation':[EDAM_OPE],
                     'input':[{'data':EDAM_DATA, 'format':[EDAM_FORMAT]}],
                     'output':[{'data':EDAM_DATA, 'format':[EDAM_FORMAT]}]}]
        self.biotool.add_functions(function)
        # Check function
        self.assertEqual(self.biotool.functions[0].operations[0].uri, EDAM_OPE['uri'])
        self.assertEqual(self.biotool.functions[0].operations[0].term, EDAM_OPE['term'])
        # Check inputs
        self.assertEqual(self.biotool.functions[0].inputs[0].data_type.uri,\
                         EDAM_DATA['uri'])
        self.assertEqual(self.biotool.functions[0].inputs[0].data_type.term, \
                         EDAM_DATA['term'])
        self.assertEqual(self.biotool.functions[0].inputs[0].formats[0].uri,\
                         EDAM_FORMAT['uri'])
        self.assertEqual(self.biotool.functions[0].inputs[0].formats[0].term, \
                         EDAM_FORMAT['term'])
        self.assertIsNone(self.biotool.functions[0].inputs[0].description)
        # Check outputs
        self.assertEqual(self.biotool.functions[0].outputs[0].data_type.uri, \
                         EDAM_DATA['uri'])
        self.assertEqual(self.biotool.functions[0].outputs[0].data_type.term, \
                         EDAM_DATA['term'])
        self.assertEqual(self.biotool.functions[0].outputs[0].formats[0].uri, \
                         EDAM_FORMAT['uri'])
        self.assertEqual(self.biotool.functions[0].outputs[0].formats[0].term, \
                         EDAM_FORMAT['term'])
        self.assertIsNone(self.biotool.functions[0].outputs[0].description)

    def test_add_topics(self):
        topics = [EDAM_TOPIC]
        self.biotool.add_topics(topics)
        self.assertEqual(self.biotool.topics[0].uri, EDAM_TOPIC['uri'])
        self.assertEqual(self.biotool.topics[0].term, EDAM_TOPIC['term'])


class TestInformations(unittest.TestCase):

    def test_init(self):
        info = biotool_model.Informations()
        self.assertListEqual(info.publications, [])
        self.assertListEqual(info.documentations, [])
        self.assertListEqual(info.contacts, [])
        self.assertListEqual(info.tool_credits, [])


class TestCredit(unittest.TestCase):

    def test_init(self):
        dict_credit = {'comment':'a_comment', 'email':'an_email', 'gridId':'a_grid_id',\
                       'name':'a_name', 'typeEntity':'a_type_entity', 'typeRole':'a_type_role', \
                       'url':'an_url', 'orcidId':'an_orcid_id'}
        credit = biotool_model.Credit(dict_credit)
        self.assertEqual(credit.comment, dict_credit['comment'])
        self.assertEqual(credit.email, dict_credit['email'])
        self.assertEqual(credit.grid_id, dict_credit['gridId'])
        self.assertEqual(credit.name, dict_credit['name'])
        self.assertEqual(credit.type_entity, dict_credit['typeEntity'])
        self.assertEqual(credit.type_role, dict_credit['typeRole'])
        self.assertEqual(credit.url, dict_credit['url'])
        self.assertEqual(credit.orcid_id, dict_credit['orcidId'])


class TestPublication(unittest.TestCase):

    def test_init(self):
        dict_publi = {'doi':'a_doi', 'pmid':'a_pm_id', 'pmcid':'a_pmc_id', 'type':'a_type'}
        publication = biotool_model.Publication(dict_publi)
        self.assertEqual(publication.doi, dict_publi['doi'])
        self.assertEqual(publication.pmid, dict_publi['pmid'])
        self.assertEqual(publication.pmcid, dict_publi['pmcid'])
        self.assertEqual(publication.type, dict_publi['type'])


class TestDocumentation(unittest.TestCase):

    def test_init(self):
        dict_doc = {'url':'one_url', 'type':'one_type', 'comment':'one comment with spaces.'}
        documentation = biotool_model.Documentation(dict_doc)
        self.assertEqual(documentation.url, dict_doc['url'])
        self.assertEqual(documentation.type, dict_doc['type'])
        self.assertEqual(documentation.comment, dict_doc['comment'])


class TestContact(unittest.TestCase):

    def test_init(self):
        dict_contact = {'email':'an_email', 'name':'a_name'}
        contact = biotool_model.Contact(dict_contact)
        self.assertEqual(contact.email, dict_contact['email'])
        self.assertEqual(contact.name, dict_contact['name'])


class TestFunction(unittest.TestCase):

    def setUp(self):
        self.function = biotool_model.Function([EDAM_TOPIC])

    def test_init(self):
        self.assertEqual(self.function.operations[0].uri, EDAM_TOPIC['uri'])
        self.assertEqual(self.function.operations[0].term, EDAM_TOPIC['term'])
        self.assertListEqual(self.function.inputs, [])
        self.assertListEqual(self.function.outputs, [])

    def test_add_inputs(self):
        inputs = [{'data':EDAM_DATA,
                   'format': [EDAM_FORMAT]}]
        self.function.add_inputs(inputs)
        self.assertEqual(self.function.inputs[0].data_type.uri, EDAM_DATA['uri'])
        self.assertEqual(self.function.inputs[0].data_type.term, EDAM_DATA['term'])
        self.assertEqual(self.function.inputs[0].formats[0].uri, EDAM_FORMAT['uri'])
        self.assertEqual(self.function.inputs[0].formats[0].term, EDAM_FORMAT['term'])
        self.assertIsNone(self.function.inputs[0].description)

    def test_add_outputs(self):
        outputs = [{'data': EDAM_DATA,
                    'format': [EDAM_FORMAT]}]
        self.function.add_outputs(outputs)
        self.assertEqual(self.function.outputs[0].data_type.uri, EDAM_DATA['uri'])
        self.assertEqual(self.function.outputs[0].data_type.term, EDAM_DATA['term'])
        self.assertEqual(self.function.outputs[0].formats[0].uri, EDAM_FORMAT['uri'])
        self.assertEqual(self.function.outputs[0].formats[0].term, EDAM_FORMAT['term'])
        self.assertIsNone(self.function.outputs[0].description)


class TestData(unittest.TestCase):

    def test_init(self):
        data_type = EDAM_DATA
        formats = [EDAM_FORMAT]
        description = 'a description of a data with spaces.'
        data = biotool_model.Data(data_type, formats, description)
        self.assertEqual(data.data_type.uri, EDAM_DATA['uri'])
        self.assertEqual(data.data_type.term, EDAM_DATA['term'])
        self.assertEqual(data.formats[0].uri, EDAM_FORMAT['uri'])
        self.assertEqual(data.formats[0].term, EDAM_FORMAT['term'])
        self.assertEqual(data.description, description)

class TestInput(TestData):

    def test_init(self):
        TestData.test_init(self)

class TestOutput(TestData):

    def test_init(self):
        TestData.test_init(self)


class TestEdam(unittest.TestCase):

    def setUp(self):
        self.edam = biotool_model.Edam(EDAM_TOPIC)

    def test_init(self):
        self.assertEqual(self.edam.uri, EDAM_TOPIC['uri'])
        self.assertEqual(self.edam.term, EDAM_TOPIC['term'])

    def test_get_edam_id(self):
        self.assertEqual(self.edam.get_edam_id(), 'topic_0091')

class TestOperation(TestEdam):

    def test_init(self):
        TestEdam.test_init(self)

class TestDataType(TestEdam):

    def test_init(self):
        TestEdam.test_init(self)

class TestFormat(TestEdam):

    def test_init(self):
        TestEdam.test_init(self)

class TestTopic(TestEdam):

    def test_init(self):
        TestEdam.test_init(self)


class TestImportJson(unittest.TestCase):

    def test_json_from_file(self):
        json_path = os.path.dirname(__file__) + '/MacSyFinder.json'
        j = main.json_from_file(json_path)
        self.assertEqual(len(j.keys()), 32)
        self.assertEqual(j['version'], '1.0.2')
        self.assertEqual(j['name'], 'MacSyFinder')
        self.assertEqual(j['owner'], 'bneron')
        self.assertEqual(j['id'], 'MacSyFinder')

    #def test_json_from_biotool(self):


class TestMainFunctions(unittest.TestCase):

    def test_json_to_biotool(self):
        json_path = os.path.dirname(__file__) + '/MacSyFinder.json'
        j = main.json_from_file(json_path)
        biot = main.json_to_biotool(j)
        # Check 3/5 arguments of biotool object
        self.assertEqual(biot.name, 'MacSyFinder')
        self.assertEqual(biot.tool_id, 'MacSyFinder')
        self.assertEqual(biot.version, '1.0.2')
        # Check few arguments from Information object
        self.assertEqual(biot.informations.contacts[0].name, 'Bertrand Néron')
        self.assertEqual(biot.informations.tool_credits[0].name, 'Institut Pasteur')
        self.assertEqual(biot.informations.documentations[0].type, \
                         'Citation instructions')
        self.assertEqual(biot.informations.publications[0].doi, \
                         'doi:10.1371/journal.pone.0110726')
        # Check few arguments from function
        self.assertEqual(biot.functions[0].operations[0].term, \
                         'Prediction and recognition (protein)')
        self.assertEqual(biot.functions[0].inputs[0].data_type.term, \
                         'Protein sequence record')
        self.assertEqual(biot.functions[0].outputs[0].data_type.term, 'Report')
        # Check few arguments from topics
        self.assertEqual(biot.topics[0].term, 'Functional genomics')


class TestGalaxyToolGen(unittest.TestCase):

    def setUp(self):
        # Create a biotool
        self.biotool = biotool_model.Biotool('a_name', 'an_id', 'a_version', 'a_description.',\
                                     'a_homepage')
        self.genxml = galaxy.GalaxyToolGen(self.biotool)

    def test_init(self):
        # Test counters
        self.assertEqual(self.genxml.input_ct, 0)
        self.assertEqual(self.genxml.output_ct, 0)
        # Copy tool to make it easier to read
        tool = self.genxml.tool
        # Test simple values of the tool
        self.assertEqual(tool.version_command, "COMMAND --version")
        # Test <tool> of the future XML
        self.assertEqual(tool.root.attrib['id'], 'an_id')
        self.assertEqual(tool.root.attrib['name'], 'a_name')
        self.assertEqual(tool.root.attrib['version'], 'a_version')
        # Test <description> of the future XML
        self.assertEqual(tool.root.find('description').text, 'a_description.')

    def test_init_existing(self):
        genxml = galaxy.GalaxyToolGen(self.biotool,
                                    existing_tool=os.path.dirname(__file__) +
                                                  '/test_write_xml.xml')
        self.assertEqual(genxml.tool.version_command, 'COMMAND --version')
        self.assertEqual(genxml.tool.command, 'COMMAND')
        self.assertEqual(genxml.tool.root.attrib['id'], 'an_id')
        self.assertEqual(genxml.tool.root.attrib['name'], 'a_name')
        self.assertEqual(genxml.tool.root.attrib['version'], 'a_version')

    def test_add_edam_topic(self):
        # Create a Topic object
        topic = biotool_model.Topic(EDAM_TOPIC)
        self.genxml.add_edam_topic(topic)
        # Test
        self.assertEqual(self.genxml.tool.edam_topics.children[0].node.text, 'topic_0091')

    def test_add_edam_operation(self):
        # Create a Operation object (Warning: EDAM_TOPIC is a topic)
        operation = biotool_model.Operation(EDAM_OPE)
        self.genxml.add_edam_operation(operation)
        # Test
        self.assertEqual(self.genxml.tool.edam_operations.children[0].node.text, 'operation_2429')

    def test_add_input_file(self):
        # Create a Input object (Warning both Type and Format will be a topic)
        an_input = biotool_model.Input(EDAM_DATA, [EDAM_FORMAT])
        self.genxml.add_input_file(an_input)
        # Copy object to test (easier to read)
        input_attrib = self.genxml.tool.inputs.children[0].node.attrib
        self.assertEqual(input_attrib['name'], 'INPUT1')
        self.assertEqual(input_attrib['format'], 'fastq')
        self.assertEqual(input_attrib['label'], EDAM_DATA['term'])
        self.assertEqual(input_attrib['type'], 'data')

    def test_add_output_file(self):
        # Create a Output object (Warning both Type and Format will be a topic)
        output = biotool_model.Output(EDAM_DATA, [EDAM_FORMAT])
        self.genxml.add_output_file(output)
        # Copy object to test
        output_attrib = self.genxml.tool.outputs.children[0].node.attrib
        self.assertEqual(output_attrib['name'], 'OUTPUT1')
        self.assertEqual(output_attrib['format'], 'fastq')
        self.assertEqual(output_attrib['from_work_dir'], 'OUTPUT1.fastq')

    def test_add_citation(self):
        # Create a Publication object
        dict_pub = {'doi':'doi:123', 'pmid':'', 'pmcid':'', 'type':'a_type'}
        publication = biotool_model.Publication(dict_pub)
        self.genxml.add_citation(publication)
        # Test
        self.assertEqual(self.genxml.tool.citations.children[0].node.text, 'doi:123')
        self.assertEqual(self.genxml.tool.citations.children[0].node.attrib['type'], 'doi')

    def test_write_xml(self):
        tmp_file = 'tmp_test_write_xml.xml'
        self.genxml.write_xml(tmp_file)
        expected_xml = os.path.dirname(__file__) + '/test_write_xml.xml'
        try:
            self.assertTrue(filecmp.cmp(expected_xml, tmp_file))
        finally:
            os.remove(tmp_file)


class TestGalaxyInfo(unittest.TestCase):

    def setUp(self):
        # Create two GalaxyInfo objects
        self.gi = edam_to_galaxy.GalaxyInfo(None)
        with requests_mock.mock() as m:
            edam_format_answer = main.json_from_file(edam_to_galaxy.LOCAL_DATA + \
                                                     '/edam_formats.json') 
            m.get('http://supergalaxy.com/api/datatypes/edam_formats', \
                  json=edam_format_answer)
            edam_data_answer = main.json_from_file(edam_to_galaxy.LOCAL_DATA + \
                                                   '/edam_data.json') 
            m.get('http://supergalaxy.com/api/datatypes/edam_data', \
                  json=edam_data_answer)
            mapping_answer = main.json_from_file(edam_to_galaxy.LOCAL_DATA + \
                                                 '/mapping.json')
            m.get('http://supergalaxy.com/api/datatypes/mapping', \
                  json=mapping_answer)
            version_answer = main.json_from_file(edam_to_galaxy.LOCAL_DATA + \
                                                 '/version.json')
            m.get('http://supergalaxy.com/api/version', \
                  json=version_answer)
            self.gi_url = edam_to_galaxy.GalaxyInfo('http://supergalaxy.com')

    def test_init(self):
        # Tests URLs
        self.assertEqual(self.gi.galaxy_url, 'https://usegalaxy.org')
        self.assertEqual(self.gi_url.galaxy_url, 'http://supergalaxy.com')
        # Tests one EDAM format
        self.assertEqual(self.gi.edam_formats['format_3579'][0], 'jpg')
        self.assertEqual(self.gi_url.edam_formats['format_3579'][0], 'jpg')
        # Tests one EDAM data
        self.assertEqual(self.gi.edam_data['data_0848'][0], 'twobit')
        self.assertEqual(self.gi_url.edam_data['data_0848'][0], 'twobit')
        # Tests class names
        self.assertEqual(self.gi.class_names['fasta'], 'galaxy.datatypes.sequence.Fasta')
        self.assertEqual(self.gi_url.class_names['fasta'], 'galaxy.datatypes.sequence.Fasta')

    def test_select_root(self):
        datatypes = ['fastq','fastq.bz2']
        for gi in [self.gi, self.gi_url]:
            root = gi.select_root(datatypes)
            self.assertEqual(root, 'fastq')


class TestEdamInfo(unittest.TestCase):

    def setUp(self):
        # Create one EdamInfo object
        self.ei = edam_to_galaxy.EdamInfo(None)

    #def test_init(self):
    #    try:
    #        self.assertEqual(len(self.ei.edam_ontology), 33192)
    #    except AssertionError:
    #        print("It is likely that EDAM has been updated.")
    #        raise

    def test_generate_hierarchy(self):
        self.ei.generate_hierarchy()
        self.assertEqual(len(self.ei.edam_data_hierarchy['data_2887']), 2)
        self.assertEqual(self.ei.edam_format_hierarchy['format_1930'][0], 'format_2182')


class TestEdamToGalaxy(unittest.TestCase):

    def setUp(self):
        # Create two EdamToGalaxy objects
        self.etog = edam_to_galaxy.EdamToGalaxy()
        try:
            self.etog_url = edam_to_galaxy.EdamToGalaxy(mapping_json='tmp_mapping.json')
        finally:
            os.remove('tmp_mapping.json')

    def test_init(self):
        for etog in [self.etog, self.etog_url]:
            self.assertEqual(etog.data_to_datatype['data_3002'], 'genetrack')
            self.assertEqual(etog.format_to_datatype['format_1930'], 'fastq')
        self.assertEqual(self.etog_url.galaxy.galaxy_url, 'https://usegalaxy.org')


class TestCwlToolGen(unittest.TestCase):

    def setUp(self):
        # Create a biotool
        self.biotool = biotool_model.Biotool('a_name', 'an_id', 'a_version', 'a_description.',
                                     'a_homepage')
        self.gencwl = cwl.CwlToolGen(self.biotool)

    def test_init(self):
        # Test counters
        self.assertEqual(self.gencwl.input_ct, 0)
        self.assertEqual(self.gencwl.output_ct, 0)
        # Copy tool to make it easier to read
        tool = self.gencwl.tool
        # Test simple values of the tool
        self.assertEqual(tool.id, "an_id")
        self.assertEqual(tool.label, "a_description.")
        self.assertEqual(tool.baseCommand, "COMMAND")
        self.assertListEqual(tool.inputs, [])
        self.assertListEqual(tool.outputs, [])

    def test_init_existing(self):
        gencwl = cwl.CwlToolGen(self.biotool,
                                existing_tool=os.path.dirname(__file__) +
                                              '/test_write_cwl.cwl')
        self.assertEqual(gencwl.tool.id, 'an_id')
        self.assertEqual(gencwl.tool.label, 'a_description.')
        self.assertEqual(gencwl.tool.cwlVersion, 'v1.0')

    def test_add_input_file(self):
        # Create a Input object (Warning both Type and Format will be a topic)
        input = biotool_model.Input(EDAM_DATA, [EDAM_FORMAT])
        self.gencwl.add_input_file(input)
        # Copy object to test (easier to read)
        input_attrib = self.gencwl.tool.inputs[0]
        self.assertEqual(input_attrib.id, 'INPUT1')
        self.assertEqual(input_attrib.type, 'File')
        self.assertEqual(input_attrib.label, EDAM_DATA['term'])
        self.assertEqual(input_attrib.format, EDAM_FORMAT['uri'])
        self.assertEqual(input_attrib.inputBinding.prefix, '--INPUT1')

    def test_add_output_file(self):
        # Create a Output object (Warning both Type and Format will be a topic)
        output = biotool_model.Output(EDAM_DATA, [EDAM_FORMAT])
        self.gencwl.add_output_file(output)
        # Copy object to test
        output_attrib = self.gencwl.tool.outputs[0]
        self.assertEqual(output_attrib.id, 'OUTPUT1')
        self.assertEqual(output_attrib.type, 'File')
        self.assertEqual(output_attrib.label, EDAM_DATA['term'])
        self.assertEqual(output_attrib.format, EDAM_FORMAT['uri'])
        self.assertEqual(output_attrib.outputBinding.glob, 'OUTPUT1.ext')

    def test_write_cwl(self):
        tmp_file = 'tmp_test_write_cwl.cwl'
        self.gencwl.write_cwl(tmp_file, 1)
        expected_cwl = os.path.dirname(__file__) + '/test_write_cwl.cwl'
        try:
            self.assertTrue(filecmp.cmp(expected_cwl, 'tmp_test_write_cwl1.cwl'))
        finally:
            os.remove('tmp_test_write_cwl1.cwl')


###########  Main  ###########

if __name__ == "__main__":
    unittest.main()
